//******************************************************************************
//GoldMap
//by BlueSonic (aka Jigglypuff)
//******************************************************************************
unit IniEditor;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  StdCtrls, ExtCtrls, Spin, People, Clipbrd, Dialogs;

type
  TIniForm = class(TForm)
    CopyMapButton: TButton;
    PasteMapButton: TButton;
    GroupBox1: TGroupBox;
    CategoryList: TListBox;
    Label17: TLabel;
    CategoryNameEdit: TEdit;
    NewCategoryButton: TButton;
    SaveCategotyButton: TButton;
    DeleteCategoryButton: TButton;
    Label16: TLabel;
    ContentsList: TListBox;
    AddContentButton: TButton;
    RemoveContentButton: TButton;
    GroupBox2: TGroupBox;
    Label2: TLabel;
    Label3: TLabel;
    EndAddressLabel: TLabel;
    Label5: TLabel;
    Label4: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    MapList: TListBox;
    NameEdit: TEdit;
    StartAddressEdit: TEdit;
    YSizeSpin: TSpinEdit;
    XSizeSpin: TSpinEdit;
    TilesetNumberSpin: TSpinEdit;
    CommentEdit: TEdit;
    SaveMapButton: TButton;
    NewMapButton: TButton;
    DeleteMapButton: TButton;
    GroupBox3: TGroupBox;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Label13: TLabel;
    Label14: TLabel;
    EventList: TListBox;
    NumberOfWarpsSpin: TSpinEdit;
    WarpsAddressEdit: TEdit;
    NumberOfSingpostsSpin: TSpinEdit;
    SingpostsAddressEdit: TEdit;
    NumberOfPeopleSpin: TSpinEdit;
    PeopleAddressEdit: TEdit;
    NewEventButton: TButton;
    SaveEventButton: TButton;
    DeleteEventButton: TButton;
    GroupBox4: TGroupBox;
    Label19: TLabel;
    PokemonList: TListBox;
    DeletePokemonButton: TButton;
    NewPokemonButton: TButton;
    SavePokemonButton: TButton;
    PokemonAddress: TEdit;
    GroupBox5: TGroupBox;
    ShowHideList: TListBox;
    ShowHideDescriptionEdit: TEdit;
    Label21: TLabel;
    procedure FormShow(Sender: TObject);
    procedure MapListClick(Sender: TObject);
    procedure SaveMapButtonClick(Sender: TObject);
    procedure NewMapButtonClick(Sender: TObject);
    procedure DeleteMapButtonClick(Sender: TObject);
    procedure EventListClick(Sender: TObject);
    procedure DeleteEventButtonClick(Sender: TObject);
    procedure SaveEventButtonClick(Sender: TObject);
    procedure NewEventButtonClick(Sender: TObject);
    procedure CategoryListClick(Sender: TObject);
    procedure SaveCategotyButtonClick(Sender: TObject);
    procedure NewCategoryButtonClick(Sender: TObject);
    procedure DeleteCategoryButtonClick(Sender: TObject);
    procedure RemoveContentButtonClick(Sender: TObject);
    procedure AddContentButtonClick(Sender: TObject);
    procedure ContentsListClick(Sender: TObject);
    procedure PokemonListClick(Sender: TObject);
    procedure SavePokemonButtonClick(Sender: TObject);
    procedure DeletePokemonButtonClick(Sender: TObject);
    procedure NewPokemonButtonClick(Sender: TObject);
    procedure ShowHideDescriptionEditChange(Sender: TObject);
    procedure ShowHideListClick(Sender: TObject);
    procedure FormDeactivate(Sender: TObject);
    procedure CopyMapButtonClick(Sender: TObject);
    procedure PasteMapButtonClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  IniForm: TIniForm;

implementation

uses Main;

{$R *.DFM}

procedure TIniForm.FormShow(Sender: TObject);
begin
MainIni.ReadSections(CategoryList.Items);
CategoryList.ItemIndex := 0;
CategoryList.OnClick(nil);
MapsIni.ReadSections(MapList.Items);
EventsIni.ReadSections(EventList.Items);
PokemonIni.ReadSections(PokemonList.Items);
ShowHideList.Items := PeopleForm.WhenBox.Items;
end;

procedure TIniForm.MapListClick(Sender: TObject);
begin
NameEdit.Text := MapList.Items[MapList.ItemIndex];
StartAddressEdit.Text := IntToHex(MapsIni.ReadInteger(NameEdit.Text,'Start Offset',0),0);
XSizeSpin.Value := MapsIni.ReadInteger(NameEdit.Text,'X Size',0);
YSizeSpin.Value := MapsIni.ReadInteger(NameEdit.Text,'Y Size',0);
EndAddressLabel.Caption := 'End Offset: '+IntToHex(XSizeSpin.Value * YSizeSpin.Value + MapsIni.ReadInteger(NameEdit.Text,'Start Offset',0) - 1,0);
TilesetNumberSpin.Value := MapsIni.ReadInteger(NameEdit.Text,'Tileset',0);
CommentEdit.Text := MapsIni.ReadString(NameEdit.Text,'Comment','');
//shwo map's event data too, if possible
EventList.ItemIndex := EventList.Items.IndexOf(NameEdit.Text);
if EventList.ItemIndex = -1 then
 begin
 //enable new event data button if there is no events
 NewEventButton.Enabled := True;
 SaveEventButton.Enabled := False;
 DeleteEventButton.Enabled := False;
 end
else
 begin
 //otherwise it's disabled
 NewEventButton.Enabled := False;
 SaveEventButton.Enabled := True;
 DeleteEventButton.Enabled := True;
 EventList.OnClick(nil);
 end;
end;

procedure TIniForm.SaveMapButtonClick(Sender: TObject);
begin
EventList.ItemIndex := EventList.Items.IndexOf(MapList.Items[maplist.itemindex]);
//if map has no events, create
if EventList.ItemIndex > -1 then
 begin
 DeleteEventButton.Click;
 NewEventButton.click;
 SaveEventButton.click;
 end;
//erase old map info
MapsIni.EraseSection(MapList.Items[MapList.ItemIndex]);
//and write new
MapsIni.Writestring(NameEdit.Text,'Start Offset','$'+StartAddressEdit.Text);
MapsIni.WriteInteger(NameEdit.Text,'X Size',XSizeSpin.Value);
MapsIni.WriteInteger(NameEdit.Text,'Y Size',YSizeSpin.Value);
MapsIni.WriteInteger(NameEdit.Text,'Tileset',TilesetNumberSpin.Value);
MapsIni.WriteString(NameEdit.Text,'Comment',CommentEdit.Text);
//set new name to maplist, in case it's changed
MapList.Items[MapList.ItemIndex] := NameEdit.Text;
end;

procedure TIniForm.NewMapButtonClick(Sender: TObject);
begin
if MapList.Items.IndexOf(NameEdit.Text) > -1 then
 begin
 MessageDlg('Map "' + NameEdit.Text + '" already exists.',mtError,[mbOk],0);
 Exit;
 end;
MapList.Items.Add(NameEdit.Text);
MapsIni.WriteInteger(NameEdit.Text,'Start Offset',StrToInt('$'+StartAddressEdit.Text));
MapsIni.WriteInteger(NameEdit.Text,'X Size',XSizeSpin.Value);
MapsIni.WriteInteger(NameEdit.Text,'Y Size',YSizeSpin.Value);
MapsIni.WriteInteger(NameEdit.Text,'Tileset',TilesetNumberSpin.Value);
MapsIni.WriteString(NameEdit.Text,'Comment',CommentEdit.Text);
//select newly added map entry
MapList.ItemIndex := MapList.Items.IndexOf(NameEdit.Text);
end;

procedure TIniForm.DeleteMapButtonClick(Sender: TObject);
begin
if MessageDlg('Are you sure that you want to delete map "' + MapList.Items[MapList.ItemIndex] + '"?',mtConfirmation,[mbYes,mbNo],0) = mrYes then
 begin
 MapsIni.EraseSection(MapList.Items[MapList.ItemIndex]);
 MapList.Items.Delete(MapList.ItemIndex);
 MapList.ItemIndex := 0;
 end;
end;

procedure TIniForm.EventListClick(Sender: TObject);
begin
//load the values from ini to edit boxes
NumberOfWarpsSpin.Value := EventsIni.ReadInteger(EventList.Items[EventList.ItemIndex],'Warps',0);
NumberOfSingpostsSpin.Value := EventsIni.ReadInteger(EventList.Items[EventList.ItemIndex],'Singposts',0);
NumberOfPeopleSpin.Value := EventsIni.ReadInteger(EventList.Items[EventList.ItemIndex],'Peoples',0);

WarpsAddressEdit.Text := IntToHex(EventsIni.ReadInteger(EventList.Items[EventList.ItemIndex],'Warp Offset',0),0);
SingpostsAddressEdit.Text := IntToHex(EventsIni.ReadInteger(EventList.Items[EventList.ItemIndex],'Singpost Offset',0),0);
PeopleAddressEdit.Text := IntToHex(EventsIni.ReadInteger(EventList.Items[EventList.ItemIndex],'People Offset',0),0);

NewEventButton.Enabled := False;
SaveEventButton.Enabled := True;
DeleteEventButton.Enabled := True;
end;

procedure TIniForm.DeleteEventButtonClick(Sender: TObject);
begin
EventsIni.EraseSection(EventList.Items[EventList.ItemIndex]);
EventList.Items.Delete(EventList.ItemIndex);
end;

procedure TIniForm.SaveEventButtonClick(Sender: TObject);
begin
EventsIni.EraseSection(EventList.Items[EventList.ItemIndex]);
EventsIni.WriteInteger(EventList.Items[EventList.ItemIndex],'Warps',NumberOfWarpsSpin.Value);
EventsIni.WriteInteger(EventList.Items[EventList.ItemIndex],'Singposts',NumberOfSingpostsSpin.Value);
EventsIni.WriteInteger(EventList.Items[EventList.ItemIndex],'Peoples',NumberOfPeopleSpin.Value);

EventsIni.WriteString(EventList.Items[EventList.ItemIndex],'Warp Offset','$'+WarpsAddressEdit.Text);
EventsIni.WriteString(EventList.Items[EventList.ItemIndex],'Singpost Offset','$'+SingpostsAddressEdit.Text);
EventsIni.WriteString(EventList.Items[EventList.ItemIndex],'People Offset','$'+PeopleAddressEdit.Text);
end;

procedure TIniForm.NewEventButtonClick(Sender: TObject);
begin
EventList.Items.Add(NameEdit.Text);
EventList.ItemIndex := EventList.Items.IndexOf(NameEdit.text);
end;

procedure TIniForm.CategoryListClick(Sender: TObject);
var
I: Integer;
S: TStringList;
begin
CategoryNameEdit.Text := CategoryList.Items[CategoryList.ItemIndex];
ContentsList.items.Clear;
S := TStringList.Create;
MainIni.ReadSection(CategoryNameEdit.Text,S);
//add the maps belonging to category
for I := 1 to S.Count do
ContentsList.Items.Add(MainIni.ReadString(CategoryNameEdit.Text,inttostr(I),''));
S.Free;
end;

procedure TIniForm.SaveCategotyButtonClick(Sender: TObject);
var I: Integer;
begin
MainIni.EraseSection(CategoryList.Items[CategoryList.ItemIndex]);
for I := 1 to ContentsList.Items.Count do
MainIni.WriteString(CategoryNameEdit.Text,inttostr(I),ContentsList.Items[I-1]);
//update name in categorylist in case changed
CategoryList.items[categorylist.itemindex] := Categorynameedit.text;
end;

procedure TIniForm.NewCategoryButtonClick(Sender: TObject);
begin
if CategoryList.Items.IndexOf(CategoryNameEdit.Text) > -1 then
 begin
 MessageDlg('Category "' + CategoryNameEdit.Text + '" already exists.',mtError,[mbOk],0);
 Exit;
 end;
CategoryList.Items.Add(CategoryNameEdit.Text);
end;

procedure TIniForm.DeleteCategoryButtonClick(Sender: TObject);
begin
MainIni.EraseSection(CategoryList.Items[Categorylist.itemindex]);
Categorylist.items.Delete(Categorylist.itemindex);
end;

procedure TIniForm.RemoveContentButtonClick(Sender: TObject);
begin
ContentsList.Items.Delete(ContentsList.ItemIndex);
end;

procedure TIniForm.AddContentButtonClick(Sender: TObject);
begin
ContentsList.Items.Add(MapList.Items[MapList.ItemIndex]);
end;

procedure TIniForm.ContentsListClick(Sender: TObject);
begin
MapList.ItemIndex := MapList.Items.IndexOf(ContentsList.Items[ContentsList.ItemIndex]);
MapList.OnClick(nil);//let maplist's OnClick load the info
end;

procedure TIniForm.PokemonListClick(Sender: TObject);
begin
PokemonAddress.Text := IntToHex(PokemonIni.ReadInteger(PokemonList.Items[PokemonList.ItemIndex],'Start Offset',0),0);
end;

procedure TIniForm.SavePokemonButtonClick(Sender: TObject);
begin
PokemonIni.WriteString(PokemonList.Items[PokemonList.ItemIndex],'Start Offset','$' + PokemonAddress.Text);
end;

procedure TIniForm.DeletePokemonButtonClick(Sender: TObject);
begin
PokemonIni.EraseSection(PokemonList.Items[PokemonList.ItemIndex]);
PokemonList.Items.Delete(PokemonList.ItemIndex);
end;

procedure TIniForm.NewPokemonButtonClick(Sender: TObject);
begin
if PokemonList.Items.IndexOf(MapList.Items[MapList.ItemIndex]) = -1 then
PokemonList.Items.Add(MapList.Items[MapList.ItemIndex])
else
MessageDlg('Wild Pokmon data for selected map already exists.',mtInformation,[mbOk],0);
end;

procedure TIniForm.ShowHideDescriptionEditChange(Sender: TObject);
begin
ShowHideList.Items[ShowHideList.ItemIndex] := ShowHideDescriptionEdit.Text;
HideShowIni.WriteString('Hidden/Shown',inttostr(ShowHideList.ItemIndex),ShowHideDescriptionEdit.Text);
PeopleForm.WhenBox.Items := ShowHideList.Items;
end;

procedure TIniForm.ShowHideListClick(Sender: TObject);
begin
ShowHideDescriptionEdit.Text := ShowHideList.Items[ShowHideList.ItemIndex];
end;

procedure TIniForm.FormDeactivate(Sender: TObject);
begin
MainForm.MainBox.Items := CategoryList.Items;
end;

procedure TIniForm.CopyMapButtonClick(Sender: TObject);
var
C: TStringList;
begin
C := TStringList.Create;
C.Add('Gold Map Clipboard Data');
C.Add(NameEdit.Text);
C.Add(StartAddressEdit.Text);
C.Add(XSizeSpin.Text);
C.Add(YSizeSpin.Text);
C.Add(TilesetNumberSpin.Text);
C.Add(CommentEdit.Text);
EventList.ItemIndex := EventList.Items.IndexOf(NameEdit.Text);
if EventList.ItemIndex <> -1 then
 begin
 C.Add(NumberOfWarpsSpin.Text);
 C.Add(NumberOfSingpostsSpin.Text);
 C.Add(NumberOfPeopleSpin.Text);
 C.Add(WarpsAddressEdit.Text);
 C.Add(SingpostsAddressEdit.Text);
 C.Add(PeopleAddressEdit.Text);
 end
else
C.Add('no events');
Clipboard.AsText := C.Text;
C.Free;
end;

procedure TIniForm.PasteMapButtonClick(Sender: TObject);
var
C: TStringList;
begin
C := TStringList.Create;
C.Text := Clipboard.AsText;
if C[0] <> 'Gold Map Clipboard Data' then
 begin
 MessageDlg('Clipboard does not contain valid map data.',mtError,[mbOK],0);
 Exit;
 end;

MapList.ItemIndex := MapList.Items.IndexOf(C[1]);
NameEdit.Text := C[1];
StartAddressEdit.Text := C[2];
XSizeSpin.Text := C[3];
YSizeSpin.Text := C[4];
TilesetNumberSpin.Text := C[5];
CommentEdit.Text := C[6];
//check if map info, which is being pasted, already exists...
if MapList.ItemIndex = -1 then
NewMapButton.Click
else
if MessageDlg('Map "' + C[1] + '" already exits. Do you want to overwrite it''s info?',mtConfirmation,[mbYes,mbNo],0) = mrNo then
Exit;
SaveMapButton.Click;
if C[7] = 'no events' then Exit;
EventList.ItemIndex := EventList.Items.IndexOf(C[1]);
NumberOfWarpsSpin.Text := C[7];
NumberOfSingpostsSpin.Text := C[8];
NumberOfPeopleSpin.Text := C[9];
WarpsAddressEdit.Text := C[10];
SingpostsAddressEdit.Text := C[11];
PeopleAddressEdit.Text := C[12];
if EventList.ItemIndex = -1 then
NewEventButton.Click else
if MessageDlg('Events for "' + C[1] + '" already exits. Do you want to overwrite event info?',mtConfirmation,[mbYes,mbNo],0) = mrNo then
Exit;
SaveEventButton.Click;
end;

end.
