//******************************************************************************
//GoldMap
//by BlueSonic (aka Jigglypuff)
//******************************************************************************
unit Map;

interface

uses ComCtrls,SysUtils,Dialogs,Classes;

type

//structure of 3-byte gameboy pointer
TGBPtr = packed record
  Bank: Byte;
  Addr: Word;
  end;

//Header 1, always before Header2 in rom
TMapHeader1 = packed record
  Header2Bank: Byte;
  Tileset: Byte;
  MapType: Byte;
  Header2Addr: Word;
  Area: Byte;
  Music: Byte;
  Unknown2: array [0..1] of Byte;
  end;

//Header 2, always in same bank after as Header1
TMapHeader2 = packed record
 BorderBlock: Byte;
 YSize: Byte;
 XSize: Byte;
 MapDataAddr: TGBPtr;
 TextAndEventBank: Byte;
 TextsAddress: Word;
 EventAddress: Word;
 Junction: array [0..1] of Byte;
 //here should be more stuff...
 end;

const
Ln = #13#10;

//table needed for encoding area names
Table: array [$00..$FF] of string[4] = (
{   0      1      2      3      4      5      6      7      8      9      A      B      C      D      E      F     }
{0}'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,
{1}'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,' '   ,
{2}'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,
{3}'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,
{4}'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,Ln    ,
{5}'#'   ,Ln+Ln ,'HIRO','GARY','POK',Ln    ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,
{6}'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,
{7}'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,''   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,' '   ,
{8}'A'   ,'B'   ,'C'   ,'D'   ,'E'   ,'F'   ,'G'   ,'H'   ,'I'   ,'J'   ,'K'   ,'L'   ,'M'   ,'N'   ,'O'   ,'P'   ,
{9}'Q'   ,'R'   ,'S'   ,'T'   ,'U'   ,'V'   ,'W'   ,'X'   ,'Y'   ,'Z'   ,'#'   ,'#'   ,':'   ,'#'   ,'#'   ,'#'   ,
{A}'a'   ,'b'   ,'c'   ,'d'   ,'e'   ,'f'   ,'g'   ,'h'   ,'i'   ,'j'   ,'k'   ,'l'   ,'m'   ,'n'   ,'o'   ,'p'   ,
{B}'q'   ,'r'   ,'s'   ,'t'   ,'u'   ,'v'   ,'w'   ,'x'   ,'y'   ,'z'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,
{C}'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,
{D}'#'   ,'''l' ,'''m' ,'''r' ,'''s' ,'''t' ,'''v' ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,
{E}''''  ,'#'   ,'#'   ,'-'   ,'#'   ,'#'   ,'?'   ,'!'   ,'.'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,'#'   ,
{F}'#'   ,' '   ,'#'   ,'#'   ,','   ,'#'   ,'0'   ,'1'   ,'2'   ,'3'   ,'4'   ,'5'   ,'6'   ,'7'   ,'8'   ,'9'
);

function GBPtrToFilePos(Bank: Byte; Addr: Word): Integer; overload;
function GBPtrToFilePos(GBPtr: TGBPtr): Integer; overload;
function GetMapHeader1Addr(MapBank: Byte; MapNum: Byte): Integer;
function GBBank(Position: Integer): Byte;
procedure ScanMapHeaders;
function GetMapHeader1(Addr: Integer): TMapHeader1;
function GetMapHeader2(MapHeader1: TMapHeader1): TMapHeader2;
procedure ReadAreaNames;
function GetAreaName(Area: Byte): string;

var
AreaNames: TStringList;

implementation

uses Main;

//converts gameboy pointer from headers into romfile offset
function GBPtrToFilePos(GBPtr: TGBPtr): Integer; overload;
begin
Result := (GBPtr.Bank * $4000) + (GBPtr.Addr - $4000);
end;

//converts rombank and, address in that bank, into romfile offset
function GBPtrToFilePos(Bank: Byte; Addr: Word): Integer; overload;
begin
Result := (Bank * $4000) + (Addr - $4000);
end;

//gives gameboy rombank number ar specified file offset
function GBBank(Position: Integer): Byte;
begin
Result := (Position div $4000);
end;

//reads & converts 2 byte pointer at Addr, referencing into specified rombank, into romfile offset
function ReadGBPtr(Addr: Integer; Bank: Byte): Integer; overload;
var
Ptr: Word;
begin
Rom.Position := Addr;
Rom.Read(Ptr,2);
Result := GBPtrToFilePos(Bank,Ptr);
end;

//reads & converts 3 byte pointer at Addr into romfile offset
function ReadGBPtr(Addr: Integer): Integer; overload;
var
Bank: Byte;
Ptr: Word;
begin
Rom.Position := Addr;
Rom.Read(Bank,2);
Rom.Read(Ptr,2);
Result := GBPtrToFilePos(Bank,Ptr);
end;

function GetMapHeader1Addr(MapBank: Byte; MapNum: Byte): Integer;
var
MapbankPtrAddr,HeaderPtrArray: Integer;
begin
MapbankPtrAddr := $940ED + ((MapBank-1) *2);//calculates address of pointer, pointing into mapbank's map header pointer array, shl = * 2?
HeaderPtrArray := ReadGBPtr(MapbankPtrAddr,$25);//reads the pointer & gets address to map header pointer array of mapbank
Result := (((MapNum-1) * 9) + HeaderPtrArray);
end;

procedure ScanMapHeaders;
var
CategoryNodes: array [0..255] of TTreeNode;

 procedure AddMap(Area,Number,Bank: Integer);
 begin
 if MainForm.SortbyArea1.Checked then
  begin
  if CategoryNodes[Area] = nil then
  CategoryNodes[Area] := MainForm.MapTree.Items.Add(nil,AreaNames[Area]);
  with MainForm.MapTree.Items.AddChild(CategoryNodes[Area],'Map ' + IntToStr(CategoryNodes[Area].Count + 1)) do
   begin
   ImageIndex    := 1;
   SelectedIndex := 1;
   Data := Pointer((Bank shl 8) or (Number));
   end;
  end
 else
  begin
  if CategoryNodes[Bank] = nil then
  CategoryNodes[Bank] := MainForm.MapTree.Items.Add(nil,'Bank ' + IntToStr(Bank));
  with MainForm.MapTree.Items.AddChild(CategoryNodes[Bank],'Map ' + IntToStr(Number) + ' - ' + AreaNames[Area]) do
   begin
   ImageIndex    := 1;
   SelectedIndex := 1;
   Data := Pointer((Bank shl 8) or (Number));
   end;
  end;
 end;

var
Bank: Byte;
W: Word;
BankAddr,NextBankAddr,I: Integer;
Node: TTreeNode;
MapHdr1: TMapHeader1;
MapHdr2: TMapHeader2;
begin
MainForm.MapTree.SortType := stNone;
MainForm.MapTree.Items.BeginUpdate;
FillChar(CategoryNodes,SizeOf(CategoryNodes),0);//make all NIL
MainForm.MapTree.Items.Clear;
MainForm.MapTree.Show;
MainForm.Splitter1.Show;
Bank := 1;
Rom.Position := $940ED + ((Bank - 1) * 2);
Rom.Read(W,2);
BankAddr := GBPtrToFilePos($25,W);
Rom.Read(W,2);
NextBankAddr := GBPtrToFilePos($25,W);
//first scan map header arrays until next map header array address is smaller then current
while (NextBankAddr > BankAddr) do
 begin
 for I := 1 to (NextBankAddr - BankAddr) div SizeOf(TMapHeader1) do
  begin
  MapHdr1 := GetMapHeader1( BankAddr + ((I - 1) * 9));
  if MapHdr1.Area < AreaNames.Count then
  AddMap(MapHdr1.Area,I,Bank)
  else
  Break;
  end;
 Inc(Bank);
 Rom.Position := $940ED + ((Bank - 1) * 2);
 Rom.Read(W,2);
 BankAddr := GBPtrToFilePos($25,W);
 Rom.Read(W,2);
 NextBankAddr := GBPtrToFilePos($25,W);
 end;
//now this will be last bank (no next bank will be calculated or processed)
for I := 1 to 255 do
 begin
 MapHdr1 := GetMapHeader1(BankAddr + ((I - 1) * 9));
 MapHdr2 := GetMapHeader2(MapHdr1);
 //check if entry is valid
 if (MapHdr2.XSize * MapHdr2.YSize <> 0) and (MapHdr1.Header2Bank = $25) and (MapHdr1.Area <= 94) then
 AddMap(MapHdr1.Area,I,Bank)
 else
 Break;//break loop if YSize is 0 or Header2 bank is not $25
 end;
MainForm.MapTree.Items.EndUpdate;
if MainForm.SortbyArea1.Checked then
MainForm.MapTree.SortType := stText;
end;

function GetMapHeader1(Addr: Integer): TMapHeader1;
begin
Rom.Position := Addr;
Rom.Read(Result,SizeOf(TMapHeader1));
end;

function GetMapHeader2(MapHeader1: TMapHeader1): TMapHeader2;
begin
Rom.Position := GBPtrToFilePos(MapHeader1.Header2Bank,MapHeader1.Header2Addr);
Rom.Read(Result,SizeOf(TMapHeader2));
end;

function GetAreaName(Area: Byte): string;
var
W: Word;
B: Byte;
I: Integer;
Capitalize: boolean;
begin
Result := '';
//are name pointer array
Rom.Position := $92388 + ((Area - 1) * 4);
Rom.Read(W,2);
Rom.Position := GBPtrToFilePos($24,W);
Rom.Read(B,1);
while B <> $50 do
 begin
 Result := Result + Table[B];
 Rom.Read(B,1);
 end;
Result := LowerCase(Result);
Capitalize := True;
for I := 1 to Length(Result) do
 begin
 if Capitalize then
  begin
  Result[I] := UpCase(Result[I]);
  Capitalize := False;
  end;
 if (Result[I] = ' ') or (Result[I] = '.') then Capitalize := True;
 end;
end;

procedure ReadAreaNames;
var
I: Integer;
begin
if AreaNames = nil then
AreaNames := TStringList.Create
else
AreaNames.Clear;
for I := 0 to 94 do
 begin
 AreaNames.Add(GetAreaName(I));
 end;
end;

end.
